/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.field.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

public class FieldDataObject extends MultiDataObject implements PropertyChangeListener {

    private InstanceContent ic;
    private Lookup intern;
    private FieldManager fieldManager;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field");

    public FieldDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(Field.MIMETYPE, true);
        
        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        fieldManager = Lookup.getDefault().lookup(FieldManager.class);
        assert fieldManager != null : "No FieldManager class found in the default lookup, unable to load the DataObject";
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
        Field field = fieldManager.load(getPrimaryFile());
        assert field != null;
        LOGGER.log(Level.FINE, "FieldDataObject loaded {0}", System.identityHashCode(field));
        ic.add(field);
        LOGGER.log(Level.FINE, "Adding new SimpleLayer in FieldDataObject lookup");
        ic.add(new SimpleLayer(field.getName()));
        field.addPropertyChangeListener(this);
        LOGGER.log(Level.FINE, "Creating new node delegate for FieldDataObject");           
        
        Node delegate = new FieldNode(super.createNodeDelegate());
        return delegate;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent arg0) {
        SaveCookie saveCookie =
                this.getLookup().
                lookup(SaveCookie.class);
        if (null == saveCookie) {
            this.ic.add(new FieldSaveCapability());
        }
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    @Override
    protected FileObject handleRename(String name) throws IOException {
        System.out.println("TEST");
        return super.handleRename(name);
    }

    
    private class FieldSaveCapability implements SaveCookie {

        @Override
        public void save() throws IOException {
            FileObject fo = FieldDataObject.this.getPrimaryFile();
            Field field = FieldDataObject.this.getLookup().
                    lookup(Field.class);
            FieldDataObject.this.fieldManager.save(field, fo);
            FieldDataObject.this.ic.remove(FieldSaveCapability.this);
        }
    }
    
    private static final class FieldNode extends FilterNode{

        public FieldNode(Node original) {
            super(original);
            assert original.getLookup().lookup(Field.class)!= null:"Should be a Field in the lookup of a FieldNode";
            //disableDelegation(DELEGATE_SET_DISPLAY_NAME);
            disableDelegation(DELEGATE_SET_NAME);
            setName(original.getLookup().lookup(Field.class).getName());
            setDisplayName(getName());
        }

//        @Override
//        public boolean canRename() {
//            System.out.println("TEST");
//            return true;
//        }
//
//        @Override
//        public String getDisplayName() {
//            return super.getDisplayName();
//        }
    }
}
